from ctranslate2.extensions import asyncio
from flask import Flask, abort, request
from tempfile import NamedTemporaryFile
from dotenv import load_dotenv
import os
from torch.functional import Tensor
import whisper
import torch
import sys
import re
from faster_whisper import WhisperModel
from test_utils import elapsed_time
from whisper_timestamped import transcribe_timestamped
from multiprocessing import Process

load_dotenv()
model_size = "small"
# tiny.en, tiny, base.en, base, small.en, small, medium.en, medium, large-v1, large-v2, large-v3, large, distil-large-v2, distil-medium.en, distil-small.en
HARPYIA_PROMPT = os.getenv("HARPYIA_PROMPT") or "спасите помогите на помощь пожар"
HARPYIA_MODEL = os.getenv("HARPYIA_MODEL") or "medium"
HARPYIA_LANGUAGE = os.getenv("HARPYIA_LANGUAGE") or "ru"

# Check if NVIDIA GPU is available
DEVICE = "cuda" if torch.cuda.is_available() else "cpu"
DEVICE = "cpu"

# Load the Whisper model:
model = WhisperModel(
    model_size,
    device=DEVICE,
    num_workers=6,
    cpu_threads=10,
    # in_memory=True,
)

app = Flask(__name__)


@app.route("/")
def hello():
    return "To recognize an audio file, upload it using a POST request with '/recognize' or '/recognize_number' route."


def recognize_files(handler_fn):
    if not request.files:
        abort(400)

    results = []
    asyncio.get_running_loop()
    for filename, handle in request.files.items():
        temp = NamedTemporaryFile()
        handle.save(temp)

        audio = prepare_file(temp.name)
        res = trans(audio)

        results.append(
            {
                "filename": filename,
                "transcript": res,
            }
        )

    print(results, file=sys.stderr)
    return {"results": results}


initprompt = [
    "один",
    "два",
    "три",
    "четыре",
    "пять",
    "шесть",
    "семь",
    "восемь",
    "девять",
    "десять",
    "одинадцать",
    "двенадцать",
    "тренадцать",
    "сто",
    "сот",
]


@elapsed_time
def trans(audio):

    segments, _ = model.transcribe(
        audio,
        language=HARPYIA_LANGUAGE,
        initial_prompt="семь сот сто",
        condition_on_previous_text=False,
        vad_filter=True,
        beam_size=5,
    )

    words = []
    for e in list(segments):
        words.append(e.text)

    return " ".join(words)


@elapsed_time
def prepare_file(filename: str):
    audio = whisper.load_audio(filename, sr=16000)
    audio = whisper.pad_or_trim(audio)
    return audio


@app.route("/recognize", methods=["POST"])
async def recognize():
    return recognize_files(lambda text: text)


@app.route("/recognize_number", methods=["POST"])
def recognize_number():
    return recognize_files(transfer_and_clean)


def transfer_and_clean(input_string):
    number_mapping = {"один": "1", "два": "2", "три": "3"}

    for word, number in number_mapping.items():
        input_string = input_string.replace(word, number)

    input_string = re.sub(r"[^\d]+", "", input_string)

    return input_string
